package com.tzld.piaoquan.ad.engine.service.score.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.tzld.piaoquan.ad.engine.commons.util.DateUtils;
import com.tzld.piaoquan.ad.engine.service.score.dto.AdPlatformBidCreativeDTO;
import com.tzld.piaoquan.ad.engine.service.score.param.BidRankRecommendRequestParam;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdItemFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.UserAdFeature;
import com.tzld.piaoquan.ad.engine.commons.score.ScoreParam;
import com.tzld.piaoquan.ad.engine.commons.score.ScorerUtils;
import com.tzld.piaoquan.ad.engine.commons.util.CommonCollectionUtils;
import com.tzld.piaoquan.ad.engine.service.score.FeatureRemoteService;
import com.tzld.piaoquan.ad.engine.service.score.RankService;
import com.tzld.piaoquan.ad.engine.service.score.convert.RequestConvert;
import com.tzld.piaoquan.ad.engine.service.score.param.RankRecommendRequestParam;
import org.joda.time.LocalTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class RankServiceImpl implements RankService {

    private final static Logger log = LoggerFactory.getLogger(RankServiceImpl.class);

    @Autowired
    FeatureRemoteService featureRemoteService;

    @Autowired
    RankServiceThompsonImpl rankServiceThompson;

    public AdRankItem adItemRank(RankRecommendRequestParam request){

        ScoreParam param= RequestConvert.requestConvert(request);
        LocalDateTime currentTime = LocalDateTime.now();
        int currentHour = currentTime.getHour();
        int dayOfWeek = currentTime.getDayOfWeek().getValue();
        param.getRequestContext().setHour(currentHour+"");
        param.getRequestContext().setWeek(dayOfWeek+"");

        UserAdFeature userAdFeature=featureRemoteService.getUserAdFeature(request.getMid());
        if(userAdFeature==null){
            userAdFeature=new UserAdFeature();
        }
        List<AdRankItem> rankItems=featureRemoteService.
                getAllAdFeatureList(
                        CommonCollectionUtils.toList(request.getAdIdList(),id->id.toString())
                );
        //不判空
        for(AdRankItem item:rankItems){
            item.setBid1(1d);
            item.setBid2(1d);
            item.setCpa(75d);
        }

        //兜底方案
        List<AdRankItem> rankResult;
        if(rankItems==null|| rankItems.size()==0){
            rankItems=new LinkedList<>();
            for(Long adId:request.getAdIdList()){
                AdRankItem item=new AdRankItem();
                item.setBid1(1d);
                item.setBid2(1d);
                item.setCpa(75d);
                item.setAdId(adId);
                item.setItemFeature(new AdItemFeature());
                rankItems.add(item);
            }
            rankResult=rankServiceThompson.rank(param, userAdFeature, rankItems,null);
        }else {

            rankResult=rank(param, userAdFeature, rankItems,ScorerUtils.BASE_CONF);
        }
        log.info("svc=adItemRank rankResult={}", JSONObject.toJSONString(rankResult));
        if (!CollectionUtils.isEmpty(rankResult)) {
            return rankResult.get(0);
        }else {
            //空返回值
            return new AdRankItem();
        }
    }

    @Override
    public AdPlatformBidCreativeDTO adBidRank(BidRankRecommendRequestParam request) {

        ScoreParam param= RequestConvert.requestConvert(request);
        UserAdFeature userAdFeature=featureRemoteService.getUserAdFeature(request.getMid());
        if(userAdFeature==null){
            userAdFeature=new UserAdFeature();
        }
        Map<String,List<AdPlatformBidCreativeDTO>> groupMap=request
                .getCreativeList()
                .stream()
                .                                  collect(Collectors.groupingBy(creativeDTO -> creativeDTO.getCreativeId()+""));

        List<AdRankItem> rankItems=featureRemoteService.
                getAllAdFeatureList(
                        new ArrayList<>(groupMap.keySet())
                );

        for(AdRankItem item:rankItems){
            try {
                AdPlatformBidCreativeDTO dto=groupMap.get(item.getAdId()+"").get(0);
                item.setBid1(dto.getBid1());
                item.setBid2(dto.getBid2());
                item.setCpa(dto.getCpa());
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        log.info("svc=adItemRank  param={}  userAdFeature={}  rankItems={}",
                JSONObject.toJSONString(param),JSONObject.toJSONString(userAdFeature),JSONObject.toJSONString(rankItems));
        List<AdRankItem> rankResult;
        if(rankItems==null|| rankItems.size()==0){
            rankItems=new LinkedList<>();
            for(AdPlatformBidCreativeDTO dto:request.getCreativeList()){
                AdRankItem item=new AdRankItem();
                item.setBid1(dto.getBid1());
                item.setBid2(dto.getBid2());
                item.setCpa(dto.getCpa());
                item.setAdId(dto.getCreativeId());
                item.setItemFeature(new AdItemFeature());
                rankItems.add(item);
            }
            rankResult=rankServiceThompson.rank(param, userAdFeature, rankItems,null);
        }else {
            rankResult=rank(param, userAdFeature, rankItems,ScorerUtils.BASE_CONF);
        }
        log.info("svc=adBidRank rankResult={}", JSONObject.toJSONString(rankResult));

        AdRankItem topItem=rankResult.get(0);

        AdPlatformBidCreativeDTO result=AdPlatformBidCreativeDTO
                .builder()
                .build();
        BeanUtils.copyProperties(topItem,result);
        result.setCreativeId(topItem.getAdId());
        result.setPctr(topItem.getCtr());
        result.setPcvr(topItem.getCvr());
        result.setCreativeCode(groupMap.get(topItem.getAdId()+"").get(0).getCreativeCode());
        if(rankResult.size()<=1){
            //经验值 待定
            result.setEcpm2(topItem.getEcpm1()*0.9);
        }else {
            result.setEcpm2(rankResult.get(1).getEcpm1());
        }
        return result;
    }



    public List<AdRankItem> rank (ScoreParam param, UserAdFeature userAdFeature, List<AdRankItem> rankItems,String configFile){
        List<AdRankItem> rankResult = ScorerUtils
                .getScorerPipeline(configFile)
                .scoring(param, userAdFeature, rankItems);
        return rankResult;
    }


}
